#!/usr/bin/php
<?php
define('PATH','/usr/local/esia/public_html/');
define('DRIVER_PATH','/usr/local/esia/public_html/plugin/virtualization/drivers/');

$_SERVER['DOCUMENT_ROOT']=PATH;
@include_once PATH.'esia.conf.php';
@include_once PATH.'plugin/virtualization/model_virtualization.php';

$DEFAULT_DATA_MAX_AGE = 5; //en secondes

$DEFAULT_OPT_TOP = 5;

$RESOURCES_TYPE = array('cluster','hypervisor','vm','storage');

$STR_STATUS=array("OK","WARNING","CRITICAL","UNKNOWN");
$INT_STATUS=array_flip($STR_STATUS);
$PRIORITIES_STATES=array("CRITICAL","WARNING","UNKNOWN","OK");

//Récupération des paramètres

//: A, authfile
$SHORT_OPTS="A:r:n:t:T:E:p:w:c:u:o:WCi:s:h";
$LONG_OPTS=array("authfile:","resource:","name:","type:",'top:','exclude:',"period:","warning::","critical::","unknown:","ok:","ret_critical",'ret_warning','interface:','speed:','help');

$options = getopt($SHORT_OPTS, $LONG_OPTS);

$o_data_max_age   = $DEFAULT_DATA_MAX_AGE;
$o_authfile	      = getParam($options,"A","authfile");
$o_resource	      = getParam($options,"r","resource");
$o_name	          = getParam($options,"n","name");
$o_type	          = getParam($options,"t","type");
//voir en fonction du o_type pour les tresholds

if( isset($options['h']) || isset($options['help']) )
{
    print_help();
}


//Vérification des paramètres avant chargement du driver
$o_resource = strtolower($o_resource);
if( empty($o_resource))
{
  exit_state('UNKNOWN',"Missing required option -r, --resource",TRUE);
}
if( !in_array($o_resource,$RESOURCES_TYPE) )
{
  exit_state('UNKNOWN',"Resource type -r, --resource $o_resource must be one of this : ".implode(',',$RESOURCES_TYPE),TRUE);
}

if( empty($o_authfile))
{
    echo "UNKNOWN : Missing required option -A, --authfile\n";
    print_section_usage($o_resource);
    exit($INT_STATUS['UNKNOWN']);
}

$authfile_path = $o_authfile;
if( !file_exists($authfile_path) ) exit_state('UNKNOWN',"Unable to found -A, --authfile $authfile_path",TRUE);

if( empty($o_type))
{
    echo "UNKNOWN : Missing required option -t, --type\n";
    print_section_usage($o_resource);
    exit($INT_STATUS['UNKNOWN']);
}
if( $o_resource != 'cluster' && $o_type != 'list' && empty($o_name))
{
  echo "UNKNOWN : Missing required option -n, --name\n";
  print_section_usage($o_resource);
  exit($INT_STATUS['UNKNOWN']);
}

//Chargement des options d'authentification
$auth_file_data = file_get_contents($authfile_path);
if( empty($auth_file_data)) exit_state('UNKNOWN',"Unable to open -A, --authfile $authfile_path");

$auth_file_data = json_decode($auth_file_data,TRUE);
if( !is_array($auth_file_data)) exit_state('UNKNOWN',"Unable to parse -A, --authfile $authfile_path");

//Chargement du driver
try
{
  $cluster = instanceDriver($auth_file_data);
}
catch(Exception $ex)
{
  exit_state('UNKNOWN',"Unable to load driver.".$ex->getMessage());
}

try{
  if( $o_resource == 'cluster' )
  {
    if( $o_type == 'cpu_usage')
    {
      $name = $cluster->getName();
      $cpu_usage = round($cluster->getCpusUsage()*100,1);
      $cpu_total = $cluster->getCpusTotal();

      $message = "Cluster $name CPU usage $cpu_usage% (TOTAL CPU : $cpu_total)";
      $ret_val = getUsagePercentRetVal($cpu_usage,$options,$reason);
      exit_state($ret_val,"$message ($reason)" );
    }
    elseif( $o_type == 'ram_usage')
    {
      $name = $cluster->getName();

      $ram_usage = $cluster->getMemUsage();
      $ram_total = $cluster->getMemTotal();
      $ram_percent = round(($ram_usage/$ram_total)*100,1);

      $unit_human = 'GB';
      $usage_human = round($ram_usage/pow(1024,3),1);
      $total_human = round($ram_total/pow(1024,3),1);

      $message = "Cluster $name Memory usage $ram_percent% ($usage_human $unit_human / $total_human $unit_human)";
      $ret_val = getUsagePercentRetVal($ram_percent,$options,$reason);
      exit_state($ret_val,"$message ($reason)|usage=$ram_usage;total=$ram_total;percent=$ram_percent" );
    }
    elseif( $o_type == 'hosts_alive')
    {
      $name = $cluster->getName();

      $host_alive = 0;
      $host_total = 0;
      foreach($cluster->getHosts() as $host)
      {
        if( $host->isAlive())
        {
          $host_alive++;
        }
        $host_total++;
      }

      $host_percent = round(($host_alive/$host_total)*100,1);

      $message = "Cluster $name Host Alive $host_percent% ($host_alive / $host_total)";
      $ret_val = getUsagePercentRetVal($host_percent,$options,$reason,TRUE);
      exit_state($ret_val,"$message ($reason)" );
    }
    elseif( $o_type == 'snapshots_min' || $o_type == 'snapshots_max' || $o_type == 'snapshots_old')
    {
        $host_list = $cluster->getHosts();
        $vm_list = array();
        foreach( $host_list as $host )
        {
            $vm_list = array_merge($vm_list,$host->getVMs());
        }

        $total =  count($vm_list);

        $ret_val = snapshotsCalc($o_type,$cluster,$vm_list,$options,$output);

        exit_state($ret_val,"They are $total VMs on cluster. $output");
    }
    else
    {
      echo "UNKNOWN : Unknown data type $o_type\n";
      print_section_usage($o_resource);
      exit($INT_STATUS['UNKNOWN']);
    }
  }
  else if( $o_resource == 'hypervisor' )
  {
    if( $o_type == 'list')
    {
      $hosts = $cluster->getHosts();
      $ret = array();
      foreach($hosts as $host)
      {
        $retRow = $host->toArray(FALSE,FALSE,TRUE);
        $disable = 'description,type,manufacturer,ha_settings';
        foreach(explode(',',$disable) as $key)
        {
          unset($retRow[$key]);
        }

        $exp = explode(' ',$retRow['uptime']);
        $retRow['uptime'] = $exp[0] . ' ' . $exp[1];

        $ret[] = $retRow;
      }
      show_result($ret,'text');
      exit($INT_STATUS['UNKNOWN']);
    }
    elseif( $o_type == 'alive')
    {
      $host = searchHost($cluster,$o_name);
      $name = $host->getName();
      $alive = $host->isAlive();

      $message = "Hypervisor $name";
      $ret_ok = FALSE;
      if( $alive )
      {
        $ret_ok = TRUE;
        $message .= " is alive.";
        getBoolRetValOption($options,$message); //exit unknown si pas retValDefault pas configuré
        exit_state('OK',$message);
      }
      else
      {
          $message .= " is down.";
      }

      $ret_val_opt = getBoolRetValOption($options,$message); //exit unknown si pas retValDefault pas configuré
      exit_state($ret_val_opt,$message);
    }
    elseif( $o_type == 'cpu_usage')
    {
      $host = searchHost($cluster,$o_name);
      $name = $host->getName();
      $cpu_usage = round($host->getCpusUsage()*100,1);
      $cpu_total = $host->getCpusTotal();

      $message = "Hypervisor $name CPU usage $cpu_usage% (TOTAL CPU : $cpu_total)";
      $ret_val = getUsagePercentRetVal($cpu_usage,$options,$reason);
      exit_state($ret_val,"$message ($reason)" );
    }
    elseif( $o_type == 'ram_usage')
    {
      $host = searchHost($cluster,$o_name);
      $name = $host->getName();

      $ram_usage = $host->getMemUsage();
      $ram_total = $host->getMemTotal();
      $ram_percent = round(($ram_usage/$ram_total)*100,1);

      $unit_human = 'GB';
      $usage_human = round($ram_usage/pow(1024,3),1);
      $total_human = round($ram_total/pow(1024,3),1);

      $message = "Hypervisor $name Memory usage $ram_percent% ($usage_human $unit_human / $total_human $unit_human)";
      $ret_val = getUsagePercentRetVal($ram_percent,$options,$reason);
      exit_state($ret_val,"$message ($reason)|usage=$ram_usage;total=$ram_total;percent=$ram_percent" );
    }
    elseif( $o_type == 'vm_usage')
    {
      $host = searchHost($cluster,$o_name);
      $name = $host->getName();

      $vms = $cluster->getVMs();
      $vm_usage=0;
      $vm_total=0;
      foreach($vms as $vm)
      {
        if( $vm->isAlive())
        {
          $vm_total++;
          $host_vm = $vm->getHost();
          if( !empty($host)) $host_vm = $host_vm->getName();
          if( $host_vm == $name)
          {
            $vm_usage++;
          }
        }
      }

      $vm_percent = round(($vm_usage/$vm_total)*100,1);

      $message = "Hypervisor $name VM alive usage $vm_percent% ($vm_usage / $vm_total)";
      $ret_val = getUsagePercentRetVal($vm_percent,$options,$reason);
      exit_state($ret_val,"$message ($reason)" );
    }
    elseif( $o_type == 'bandwidth')
    {
      $o_interface = getParam($options,"i","interface");
      if( $o_interface == 'all' )
      {
          $o_interface = NULL;
      }

      $o_speed = getParam($options,"s","speed",'auto');

      $host = searchHost($cluster,$o_name);
      $name = $host->getName();

      $ifaces = $host->getNetInterfaces();

      $ifaces_ret = NULL;
      if( $o_interface == NULL)
      {
          $ifaces_ret = $ifaces;
          unset($ifaces_ret['global']);
          $ifaces_ret = array_keys($ifaces_ret);
      }
      else
      {
          if( !isset($ifaces[$o_interface]))
          {
              exit_state('UNKNOWN',"Unknown interface name $o_interface");
          }
          $ifaces_ret = array($o_interface);
      }

      $speed = $o_speed;
      $message = '';
      $nb = 0;
      $ret_global = "OK";
      $if_default_speed = FALSE;

      foreach($ifaces_ret as $iface)
      {
          //Set speed
          if( $speed == 'auto')
          {
              if( $iface == 'global')
              {

                  $ifaces_keys = $ifaces;
                  unset($ifaces_keys['global']);
                  $speed = 0;
                  foreach($ifaces_keys as $value)
                  {
                      if( isset($value['speed'] ))
                      {
                          $speed+=$value['speed'];
                      }
                      else
                      {
                          //exit_state('UNKNOWN',"Unable to get speed of iface $key. You must set it with parameter -s, --speed");
                          $speed = 1000;
                          $if_default_speed = TRUE;
                      }
                  }
              }
              else
              {
                  if( isset($ifaces[$iface]['speed'] ))
                  {
                      $speed=$ifaces[$iface]['speed'];
                  }
                  else
                  {
                      //exit_state('UNKNOWN',"Unable to get iface speed. You must set it with parameter -s, --speed");
                      $speed = 1000;
                      $if_default_speed = TRUE;
                  }
              }
          }


           //Set message and ret
           if( $ifaces[$iface]['tx'] === NULL || $ifaces[$iface]['rx'] === NULL )
           {
               continue;
           }
           $tx = $ifaces[$iface]['tx'];
           $rx = $ifaces[$iface]['rx'];

           $rrds = "data:in=$rx"."data:out=$tx";

           $tx = round($tx/1000/1000,3);
           $rx = round($rx/1000/1000,3);

           $tx_p = round($tx/$speed*100,1);
           $rx_p = round($rx/$speed*100,1);

           if( $tx_p > $rx_p)
           {
               $max_p = $tx_p;
           }
           else
           {
               $max_p = $rx_p;
           }
           $ret_val = getUsagePercentRetVal($max_p,$options,$reason);
           if(!$if_default_speed)
           {
             $message .= "$ret_val : Hypervisor $name iface $iface in=$rx Mb/s out=$tx Mb/s speed=$speed Mb/s ($max_p%$reason)\n";
           }
           else
           {
              $message .= "$ret_val : Hypervisor $name iface $iface in=$rx Mb/s out=$tx Mb/s speed=$speed Mb/s (speed recovery failed, use default speed) ($max_p%$reason)\n";
           }
           $ret_global = calcul_state($ret_global,$ret_val);
           $nb++;
      }
      if( $nb == 0 )
      {
          exit_state('UNKNOWN',"Unable to get tx/rx data for iface $o_interface. Use 'global' in place");
      }
      else if( $nb == 1)
      {
        $message = trim($message) . "|$rrds";
      }
      echo trim($message)."\n";
      exit($ret_global);
    }
    elseif( $o_type == 'snapshots_min' || $o_type == 'snapshots_max' || $o_type == 'snapshots_old' )
    {
        $host = searchHost($cluster,$o_name);
        $name = $host->getName();

        $vm_list = $host->getVMs();

        $tmp = array();
        foreach($vm_list as $vm)
        {
            if( $vm->getState() != 'running' )
            {
                $tmp[] = $vm;
            }
        }
        $vm_list = $tmp;
        $total =  count($vm_list);

        $ret_val = snapshotsCalc($o_type,$cluster,$vm_list,$options,$output);

        exit_state($ret_val,"They are $total VMs (RUNNING only) on this host ($name). $output");
    }
    else
    {
        echo "UNKNOWN : Unknown data type $o_type\n";
        print_section_usage($o_resource);
        exit($INT_STATUS['UNKNOWN']);
    }
  }
  else if( $o_resource == 'storage')
  {
    if( $o_type == 'list')
    {
      $storages = $cluster->getStorages();
      $ret = array();
      foreach($storages as $sr)
      {
        $retRow = $sr->toArray(TRUE,FALSE,TRUE);
        $disable = 'description,type,manufacturer,ha_settings';
        $disable = '';
        foreach(explode(',',$disable) as $key)
        {
          unset($retRow[$key]);
        }
        $ret[] = $retRow;
      }
      show_result($ret,'text');
      exit($INT_STATUS['UNKNOWN']);
    }
    elseif( $o_type == 'hosts_connected')
    {
      $storage = searchStorage($cluster,$o_name);
      $name = $storage->getName();
      $hosts = array();
      foreach($storage->getHosts() as $host)
      {
        $hosts[] = $host->getName();
      }
      $connected = count($hosts);
      $total = count($cluster->getHosts());

      $hosts = implode(',',$hosts);
      $is_shared = $storage->isShared();

      $is_ok = ($connected == $total || (!$is_shared && $connected = 1));

      if( $is_shared) $type = 'Shared';
      else            $type = 'Local';

      if( $is_ok )
      {
        $message = "$type storage $name available on hosts ($hosts)";
        exit_state($INT_STATUS['OK'],$message);
      }
      else
      {
        $message = "$type storage $name available on only $connected / $total ($hosts)";
      }
      $ret_val_opt = getBoolRetValOption($options,$message); //exit unknown si pas retValDefault pas configuré
      exit_state($ret_val_opt,$message);
    }
    elseif( $o_type == 'disk_usage')
    {
      $storage = searchStorage($cluster,$o_name);
      $name = $storage->getName();

      $hosts = array();
      foreach($storage->getHosts() as $host)
      {
        $hosts[] = $host->getName();
      }
      $hosts = implode(',',$hosts);

      $disk_usage = $storage->getSizeUsed();
      $disk_total = $storage->getSizeTotal();
      $disk_percent = round(($disk_usage/$disk_total)*100,1);

      $unit_human = 'GB';
      $usage_human = round($disk_usage/pow(1024,3),1);
      $total_human = round($disk_total/pow(1024,3),1);

      $message = "Storage $name on ($hosts) Disk usage $disk_percent% ($usage_human $unit_human / $total_human $unit_human)";
      $ret_val = getUsagePercentRetVal($disk_percent,$options,$reason);
      exit_state($ret_val,"$message ($reason)|usage=$disk_usage;total=$disk_total;percent=$disk_percent" );


      $message = "VM $name CPU usage $cpu_usage% (TOTAL CPU : $cpu_total)";
      $ret_val = getUsagePercentRetVal($cpu_usage,$options,$reason);
      exit_state($ret_val,"$message ($reason)" );
    }
    else
    {
        echo "UNKNOWN : Unknown data type $o_type\n";
        print_section_usage($o_resource);
        exit($INT_STATUS['UNKNOWN']);
    }
  }
  else if( $o_resource == 'vm' )
  {
    if( $o_type == 'list')
    {
      $vms = $cluster->getVMs();
      $ret = array();
      foreach($vms as $vm)
      {
        $retRow = $vm->toArray(FALSE,FALSE,TRUE);
        $disable = 'description,type,manufacturer,ha_settings';
        foreach(explode(',',$disable) as $key)
        {
          unset($retRow[$key]);
        }

        $exp = explode(' ',$retRow['uptime']);
        $retRow['uptime'] = $exp[0] . ' ' . $exp[1];

        $ret[] = $retRow;
      }
      show_result($ret,'text');
      exit($INT_STATUS['UNKNOWN']);
    }
    elseif( $o_type == 'state')
    {
      $o_warning        = getParam($options,"w","warning");
      $o_critical       = getParam($options,"c","critical");

      $o_unknown        = getParam($options,"u","unknown");
      $o_ok             = getParam($options,"o","ok");

      $vm = searchVM($cluster,$o_name);

      $name = $vm->getName();
      $running_state = $vm->getState();
      $isLocked = $vm->isLocked();
      $lock_status = 'N/A';
      if($isLocked)
      {
        $lock_status = $vm->getLockStatus();
      }

      $treshold_string = array('CRITICAL'=>$o_critical,'WARNING'=>$o_warning,'UNKNOWN'=>$o_unknown,'OK'=>$o_ok);

      $ret_val = 'UNKNOWN';
      $message = "VM $name is $running_state and locked($lock_status)";
      foreach($treshold_string as $treshold_type => $treshold_search)
      {
        $treshold_search = explode(',',$treshold_search);
        if( in_array($running_state,$treshold_search) )
        {
          $message = "VM $name is $running_state";
          $ret_val = $treshold_type;
          break;
        }
        if( $isLocked &&  in_array("locked",$treshold_search))
        {
            $message = "VM $name is locked($lock_status)";
            $ret_val = $treshold_type;
            break;
        }
      }
      exit_state($ret_val,$message);
    }
    elseif( $o_type == 'ha_enabled')
    {
      $vm = searchVM($cluster,$o_name);
      $name = $vm->getName();
      $ha = $vm->isHA();
      $alive = $vm->isAlive();

      $message = "VM $name";
      $ret_ok = FALSE;
      if( $ha && $alive )
      {
        $ret_ok = TRUE;
        $message .= " is alive and HA enabled.";
        getBoolRetValOption($options,$message); //exit unknown si pas retValDefault pas configuré
        exit_state('OK',$message);
      }
      elseif(!$alive){
        $message.= " isn't running.";
      }
      elseif( $alive && !$ha)
      {
        $message.= " is running but ha disabled.";
      }

      $ret_val_opt = getBoolRetValOption($options,$message); //exit unknown si pas retValDefault pas configuré
      exit_state($ret_val_opt,$message);
    }
    elseif( $o_type == 'agent_enabled')
    {
      $vm = searchVM($cluster,$o_name);
      $name = $vm->getName();
      $agent = $vm->isAgent();

      $message = "VM $name";
      $ret_ok = FALSE;
      if( $agent )
      {
        $ret_ok = TRUE;
        $message .= " agent is enabled.";
        getBoolRetValOption($options,$message); //exit unknown si pas retValDefault pas configuré
        exit_state('OK',$message);
      }
      else
      {
          $message .= " agent is disabled.";
      }

      $ret_val_opt = getBoolRetValOption($options,$message); //exit unknown si pas retValDefault pas configuré
      exit_state($ret_val_opt,$message);
    }
    elseif( $o_type == 'cpu_usage')
    {
      $vm = searchVM($cluster,$o_name);
      $name = $vm->getName();
      $cpu_usage = round($vm->getCpusUsage()*100,1);
      $cpu_total = $vm->getCpusTotal();

      $message = "VM $name CPU usage $cpu_usage% (TOTAL CPU : $cpu_total)";
      $ret_val = getUsagePercentRetVal($cpu_usage,$options,$reason);
      exit_state($ret_val,"$message ($reason)" );
    }
    elseif( $o_type == 'ram_usage')
    {
      $vm = searchVM($cluster,$o_name);
      $name = $vm->getName();

      $ram_usage = $vm->getMemUsage();
      $ram_total = $vm->getMemTotal();
      $ram_percent = round(($ram_usage/$ram_total)*100,1);

      $unit_human = 'GB';
      $usage_human = round($ram_usage/pow(1024,3),1);
      $total_human = round($ram_total/pow(1024,3),1);

      $message = "VM $name Memory usage $ram_percent% ($usage_human $unit_human / $total_human $unit_human)";
      $ret_val = getUsagePercentRetVal($ram_percent,$options,$reason);
      exit_state($ret_val,"$message ($reason)|usage=$ram_usage;total=$ram_total;percent=$ram_percent" );
    }
    elseif( $o_type == 'snapshots_min')
    {
        $value = 0;
        $output = '';

        $o_period = getParam($options,'p','period');
        $ret_val = snapshots_min($cluster,$o_name,$o_period,$options,$value,$output);
        exit_state($ret_val, $output );
    }
    elseif( $o_type == 'snapshots_max')
    {
        $value = 0;
        $output = '';

        $ret_val = snapshots_max($cluster,$o_name,$options,$value,$output);
        exit_state($ret_val, $output );
    }
    elseif( $o_type == 'snapshots_old')
    {
        $value = 0;
        $output = '';

        $ret_val = snapshots_oldest($cluster,$o_name,$options,$value,$output);
        exit_state($ret_val, $output );
    }
    else
    {
        echo "UNKNOWN : Unknown data type $o_type\n";
        print_section_usage($o_resource);
        exit($INT_STATUS['UNKNOWN']);
    }
  }
}
catch(Exception $ex)
{
  exit_state('UNKNOWN','Unable to receive DATA : '.$ex->getMessage());
}

function getUsageMinRetVal($value,$options,&$reason)
{
  $o_warning  = getParam($options,'w','warning');
  $o_critical = getParam($options,'c','critical');

  if( $o_warning == NULL || $o_critical == NULL ) exit_state('UNKNOWN',"You forget to specify warning or critical treshold");

  if( $value < $o_critical )
  {
    $reason = "<$o_critical";
    return 'CRITICAL';
  }
  if( $value < $o_warning )
  {
    $reason = "<$o_warning";
    return 'WARNING';
  }
  else
  {
    $reason = ">=$o_warning";
    return 'OK';
  }
}

function getUsageMaxRetVal($value,$options,&$reason)
{
  $o_warning  = getParam($options,'w','warning');
  $o_critical = getParam($options,'c','critical');

  if( $o_warning == NULL || $o_critical == NULL ) exit_state('UNKNOWN',"You forget to specify warning or critical treshold");
  if( $o_warning > $o_critical)  exit_state('UNKNOWN',"-w, --warning treshold can't be greater than critical($o_critical)");

  if( $value > $o_critical )
  {
    $reason = ">$o_critical";
    return 'CRITICAL';
  }
  if( $value > $o_warning )
  {
    $reason = ">$o_warning";
    return 'WARNING';
  }
  else
  {
    $reason = "<=$o_warning";
    return 'OK';
  }
}

function getUsagePercentRetVal($value,$options,&$reason,$reverse=FALSE)
{
  $o_warning  = getParam($options,'w','warning');
  $o_critical = getParam($options,'c','critical');

  if( $o_warning == NULL || $o_critical == NULL ) exit_state('UNKNOWN',"You forget to specify warning or critical treshold");
  if( $o_critical > 100 ) exit_state('UNKNOWN',"-c, --critical treshold can't be greater than 100");
  if( $o_warning > 100 ) exit_state('UNKNOWN',"-w, --warning treshold can't be greater than 100");

  if( !$reverse)
  {
    if( $o_warning > $o_critical)  exit_state('UNKNOWN',"-w, --warning treshold can't be greater than critical($o_critical)");

    if( $value > $o_critical )
    {
      $reason = ">$o_critical%";
      return 'CRITICAL';
    }
    if( $value > $o_warning )
    {
      $reason = ">$o_warning%";
      return 'WARNING';
    }
    else
    {
      $reason = "<=$o_warning%";
      return 'OK';
    }
  }
  else
  {
    if( $o_warning < $o_critical)  exit_state('UNKNOWN',"-w, --warning treshold can't be lower than critical($o_critical)");

    if( $value < $o_critical )
    {
      $reason = "<$o_critical%";
      return 'CRITICAL';
    }
    if( $value < $o_warning )
    {
      $reason = "<$o_warning%";
      return 'WARNING';
    }
    else
    {
      $reason = ">$o_warning%";
      return 'OK';
    }
  }
}

function getBoolRetValOption($options,$message)
{
    if( isset($options['C']) || isset($options['ret_critical']) )
    {
      $ret_val = 'CRITICAL';
    }
    elseif( isset($options['W']) || isset($options['ret_warning'] ))
    {
      $ret_val = 'WARNING';
    }
    else
    {
      $ret_val = 'UNKNOWN';
      $message.= " return value is not set.";
      exit_state($ret_val,$message);
    }
    return $ret_val;
}

function searchStorage($cluster,$o_name)
{
  $storages = $cluster->getStorages();
  $found = array();
  foreach($storages as $sr)
  {
    if( $o_name == $sr->getID() )
    {
      $found[] = $sr;
    }
    else if ($o_name == $sr->getName() )
    {
      $found[] = $sr;
    }
    else if(!$sr->isShared()){
      $exp = explode('/',$o_name,2);
      if ($exp == 2)
      {
        $sr_host = $sr->getHosts();
        if( !empty($sr_host))
        {
          $sr_host = $sr_host->getName();
        }
        if( $exp[0] == $sr_host && $exp[1] == $sr->getName())
        {
          $found[] = $sr;
        }
      }
    }
  }
  if( count($found) > 2)
  {
    exit_state('UNKNOWN',"Mutliples storages are name -n, --name $o_name");
  }
  $found = array_shift($found);
  return $found;
}

function snapshotsCalc($type,$cluster,$vm_list,$options,&$output)
{
    global $INT_STATUS;
    global $DEFAULT_OPT_TOP;
    $top = getParam($options,'T','top',$DEFAULT_OPT_TOP);
    $exclude = getParam($options,'E','exclude',NULL);

    if( empty($exclude) )
    {
        $exclude = array();
    }
    else
    {
        $exclude = explode(',',$exclude);
    }

    $states_count = array(0,0,0,0);
    $top_list = array();

    $ret_val = 'OK';
    foreach($vm_list as $vm)
    {
        $vm_name = $vm->getName();
        $exclude_vm = FALSE;
        foreach($exclude as $ex)
        {
            if ( preg_match("/$ex/",$vm_name))
            {
                $exclude_vm = TRUE;
            }
        }
        if ( $exclude_vm )
        {
            continue;
        }

        $value = 0;
        $output_vm = '';

        if( $type == 'snapshots_min' )
        {
              $o_period = getParam($options,'p','period');
              $ret_vm = snapshots_min($cluster,$vm->getName(),$o_period,$options,$value,$output_vm);
        }
        else if( $type == 'snapshots_max' )
        {
              $ret_vm = snapshots_max($cluster,$vm->getName(),$options,$value,$output_vm);
        }
        else
        {
              $ret_vm = snapshots_oldest($cluster,$vm->getName(),$options,$value,$output_vm);
        }
        $states_count[$INT_STATUS[$ret_vm]] ++;
        $ret_val = calcul_state($ret_val,$ret_vm);

        if( !isset($top_list[$value]) )
        {
            $top_list[$value] = array();
        }
        $top_list[$value][] = "$ret_vm : $output_vm";
    }

    if( $type == 'snapshots_min' )
    {
          ksort($top_list);
    }
    else
    {
          krsort($top_list);
    }
    $i=0;
    $output = '';
    foreach($top_list as $top_value)
    {
        foreach( $top_value as $row )
        {
            if( $i >= $top){
                break;
            }
            $output .= $row . "\n";
            $i++;
        }
        if( $i >= $top){
            break;
        }
    }

    $output = sprintf("%d OK, %d WARNING, %d CRITICAL, %d UNKNOWN. Top $top :\n%s",$states_count[0],$states_count[1],$states_count[2],$states_count[3],$output);

    return $ret_val;
}

function snapshots_oldest($cluster,$o_name,$options,&$value,&$output)
{
    $vm = searchVM($cluster,$o_name);
    $name = $vm->getName();

    $snapshot_list = $vm->getSnapshots();
    $now = time();
    $min_time = $now;

    foreach($snapshot_list as $snap)
    {
        if( $snap['time'] < $min_time )
        {
            $min_time = $snap['time'];
        }
    }

    $age = $now - $min_time;
    $days = round($age /= 86400,1);

    $message = "VM $name oldest snapshot age is $days days";
    $ret_val = getUsageMaxRetVal($days,$options,$reason);
    $output = "$message ($reason)";
    $value = $days;
    return $ret_val;
}

function snapshots_max($cluster,$o_name,$options,&$value,&$output)
{
    $vm = searchVM($cluster,$o_name);
    $name = $vm->getName();

    $snapshot_list = $vm->getSnapshots();
    $nb_snap = 0;

    foreach($snapshot_list as $snap)
    {
        $nb_snap++;
    }

    $message = "VM $name has $nb_snap snapshot";
    $ret_val = getUsageMaxRetVal($nb_snap,$options,$reason);
    $output = "$message ($reason)";
    $value = $nb_snap;
    return $ret_val;
}

function snapshots_min($cluster,$o_name,$o_period,$options,&$value,&$output)
{
    $vm = searchVM($cluster,$o_name);
    $name = $vm->getName();

    $snapshot_list = $vm->getSnapshots();
    $nb_snap = 0;

    $o_period = getParam($options,'p','period');

    if( $o_period != NULL )
    {
        $interval = $o_period * 86400; //3600s * 24
        $since = time() - $interval;
    }
    else
    {
        $since = 0;
    }

    foreach($snapshot_list as $snap)
    {
        if( $snap['time'] > $since )
        {
            $nb_snap++;
        }
    }

    $message = "VM $name has $nb_snap snapshots";
    if( $since > 0 )
    {
        $message.= " since $o_period days";
    }

    $ret_val = getUsageMinRetVal($nb_snap,$options,$reason);
    $output = "$message ($reason)";
    $value = $nb_snap;
    return $ret_val;
}

function searchHost($cluster,$o_name)
{
  $host = $cluster->getHostByName($o_name);
  if(empty( $host ))
  {
    exit_state('UNKNOWN',"Host $o_name not found");
  }
  return $host;
}

function searchVM($cluster,$o_name)
{
  $vm = $cluster->getVM($o_name);
  if( empty($vm) )
  {
    $vm = $cluster->getVMByName($o_name);
    if( count($vm) > 2)
    {
      exit_state('UNKNOWN',"Multiples VM are name -n, --name $o_name");
    }
    $vm = array_shift($vm);
  }
  if(empty( $vm ))
  {
    exit_state('UNKNOWN',"VM -n, --name $o_name not found");
  }
  return $vm;
}

function show_result($data,$format){
	if( empty($data )){
		$data = array();
	}

	if( $format == 'json' ){
		echo json_encode($data)."\n";
	}
	else if( $format == 'json-pretty' ){
		echo json_encode($data,JSON_PRETTY_PRINT)."\n";
	}
	else if( $format == 'text' ){
		if( !isset($data[0]) ){
			$keys = array_keys($data);
			$size_keys = getArrayBiggestWordSize($keys);

			foreach($data as $key => $row){
				echo str_pad($key,$size_keys) . " : ";
				echo $row . "\n";
			}
		}
		else{
			$byKeysArr = array();
			foreach($data as $row){
				foreach($row as $key => $field){
					$byKeysArr[$key][] = $field;
				}
			}

			$size_col = array();
			foreach($byKeysArr as $key => $arr){
				$arr[] = $key;
				$size_col[$key] = getArrayBiggestWordSize($arr);
			}

			$keys = array_keys($byKeysArr);
			foreach( $keys as $th ){
				echo str_pad($th,$size_col[$th]). ' ';
			}
			echo "\n";

			foreach( $data as $key => $row ){
				foreach($row as $th => $td){
					echo str_pad($td,$size_col[$th]). ' ';
				}
				echo "\n";
			}
		}
	}
	else{
		echo "ERROR : Unknown outpout format '$format'\n\n";
    exit($INT_STATUS['UNKNOWN']);
	}
}

function getArrayBiggestWordSize($arr){
	$biggest_size = 0;
        foreach($arr as $word){
                $cur_size = strlen($word);
                if( $biggest_size < $cur_size){
                        $biggest_size = $cur_size;
                }
        }
	return $biggest_size;
}

function instanceDriver($settings)
{
    $drv_name = $settings['driver'];

    $drivers['proxmox']['file'] = 'proxmox/Proxmox.class.php';
    $drivers['proxmox']['class'] = 'ProxmoxCluster';
    $drivers['xenserver']['file'] = 'xenserver/XenServer.class.php';
    $drivers['xenserver']['class'] = 'XenServerCluster';

    if( !isset($drivers[$drv_name]['file']) )
    {
        throw new Exception("Driver $drv_name unknown");
    }

    $oldpath = get_include_path();
    set_include_path(DRIVER_PATH);
    include_once($drivers[$drv_name]['file']);
    set_include_path($oldpath);

    $className = $drivers[$drv_name]['class'];
    $clusterClass = new $className($settings);
    return $clusterClass;
}

function getParam($options,$name1,$name2,$default=null){
        if( isset( $options[$name1] ) ) return $options[$name1];
        if( isset( $options[$name2] ) ) return $options[$name2];
        return $default;
}

function exit_state($state,$msg=null,$usage=false){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];

        $ret=$state;
        if( is_string($state) ) $ret=$INT_STATUS[strtoupper($state)];
        else $state=$STR_STATUS[$ret];

        if( $msg != null )
        {
          $msg=trim($msg);
          echo "$state : $msg\n";
        }
        if( $usage ) print_usage(true);
        exit($ret);
}

function calcul_state($a,$b=null){
        $STR_STATUS=$GLOBALS["STR_STATUS"];
        $INT_STATUS=$GLOBALS["INT_STATUS"];
        $PRIORITIES_STATES=$GLOBALS["PRIORITIES_STATES"];

        if(is_string($a) ) $a=$INT_STATUS[strtoupper($a)];
        if(is_string($b) ) $b=$INT_STATUS[strtoupper($b)];

        foreach($PRIORITIES_STATES as $value){
                $value = $INT_STATUS[$value];

                if( $value == $a || $value == $b ) return $value;
        }
        return $INT_STATUS["UNKNOWN"];
}

function print_usage(){
	$script = basename(__FILE__);
	$sections = explode(',','cluster,hypervisor,storage,vm');

	$size_section = getArrayBiggestWordSize($sections);

	$left = 'usage : ';
	$size_left = strlen($left);
	foreach($sections as $section){
		echo str_pad($left,$size_left) . "$script -A <authfile> -r " . str_pad($section,$size_section) . " [OPTIONS]\n";
		$left = '';
	}
	echo "\n";
	echo str_pad($left,$size_left) . "$script --help for more help\n";
}

function print_section_usage($section){
	$script = basename(__FILE__);
	if( $section == 'cluster' ){
		$actions = explode(',','cpu_usage,ram_usage,host_alive,snapshots_min,snapshots_max,snapshots_old');

		$options['cpu_usage'] 	= '-w <percent_max> -c <percent_max>';
		$options['ram_usage'] 	= '-w <percent_max> -c <percent_max>';
		$options['host_alive'] 	= '-w <percent_min> -c <percent_min>';
        $options['snapshots_min']   = '-c <min_snap> -w <min_snap> [-p <n_days>] [-T <top>] [-T <exclude_list>]';
        $options['snapshots_max']   = '-w <max_snap> -c <max_snap> [-T <top>] [-T <exclude_list>]';
        $options['snapshots_old']   = '-w <max_days> -c <max_days> [-T <top>] [-T <exclude_list>]';
	}
	elseif( $section == 'hypervisor' ){
		$actions = explode(',','list,alive,cpu_usage,ram_usage,vm_usage,bandwidth,snapshots_min,snapshots_max,snapshots_old');
        $options['alive'] 	    = '[-W | -C]';
        $options['cpu_usage'] 	= '-n <hypervisor> -w <percent_max> -c <percent_max>';
		$options['ram_usage'] 	= '-n <hypervisor> -w <percent_max> -c <percent_max>';
        $options['vm_usage'] 	= '-n <hypervisor> -w <percent_max> -c <percent_max>';
		$options['bandwidth'] 	= '-n <hypervisor> -w <percent_max> -c <percent_max>';
        $options['snapshots_min']   = '-n <hypervisor> -c <min_snap> -w <min_snap> [-p <n_days>] [-T <exclude_list>]';
        $options['snapshots_max']   = '-n <hypervisor> -w <max_snap> -c <max_snap> [-T <top>] [-T <exclude_list>]';
        $options['snapshots_old']   = '-n <hypervisor> -w <max_days> -c <max_days> [-T <top>] [-T <exclude_list>]';
	}
	elseif( $section == 'storage' ){
		$actions = explode(',','list,hosts_connected,disk_usage');
		$options['hosts_connected'] = '-n <storage> [-W | -C]';
		$options['disk_usage'] 	    = '-n <storage> -w <percent_max> -c <percent_max>';
	}
	elseif( $section == 'vm' ){
		$actions = explode(',','list,state,ha_enabled,cpu_usage,ram_usage,snapshots_min,snapshots_max,snapshots_old');
        $options['state']           = '-n <vm> -o <list> -w <list> -c <list>';
        $options['ha_enabled']      = '-n <vm> [-W | -C]';
        $options['cpu_usage']       = '-n <vm> -w <percent_max> -c <percent_max>';
		$options['ram_usage']       = '-n <vm> -w <percent_max> -c <percent_max>';
        $options['vm_usage'] 	    = '-n <vm> -w <percent_max> -c <percent_max>';
		$options['snapshots_min']   = '-n <vm> -c <min_snap> -w <min_snap> [-p <n_days>]';
        $options['snapshots_max']   = '-n <vm> -w <max_snap> -c <max_snap>';
        $options['snapshots_old']   = '-n <vm> -w <max_days> -c <max_days>';
	}

    $size_action = getArrayBiggestWordSize($actions);

    $left = 'usage : ';
    $size_left = strlen($left);

    foreach($actions as $action){
        echo str_pad($left,$size_left) . "$script -A <authfile> -r $section -t " . str_pad($action,$size_action);
        if( isset($options[$action] ) )
        {
            echo " ".$options[$action];
        }
        echo "\n";
        $left = '';
    }
    echo "\n";
    echo str_pad($left,$size_left) . "$script -r $section --help for more help\n";
}

function print_help()
{
    $INT_STATUS=$GLOBALS["INT_STATUS"];
    print_usage();
?>

Main Options :
    -A, --authfile=<file_path>      Full path to authfile
    -r, --resource=<resource_key>   Resource to monitor (cluster,hypervisor,storage or vm)
    -t, --type=<data_type>          Data to monitor (cpu_usage, host_alive, bandwidth, ...). See below section for more details
    -n, --name=<resource_selector>  Required for all resources except 'cluster'. ID or name of item (vm id, storage name, ...)

Another Options :
    -p, --period=<n_days>           Days number max for creation age (for snapshot min research).
    -W, --ret_warning               When plugin check yes/no state. Return Warning or OK
    -C, --ret_critical              When plugin check ues/no state. Return Critical or OK
    -o, --ok=<treshold>             Ok treshold value. Only for list of string.
    -w, --warning=<treshold>        Warning treshold value
    -c, --critical=<treshold>       Critical treshold value
    -T, --top=<top>                 Number of line/items to print to top list.
    -E, --exclude=<regex_list>    For exclude, ignore items. With regex.

Tresholds type :
    percent_min     Min treshold in percent. If reached, return Warning or critical
    percent_max     Max treshold in percent. If reached, return Warning or critical
    min_snap        Min value for treshold. This is a number of snapshots to have.
    max_snap        Max value for treshold. This is a number of snapshots to have.
    max_days        Max days for treshold. Snapshots can't be older than this date.
    list            List of state that item type data can have. (running, stopped, ...). -o, --ok should be use

Data type :
    list            Print a complete list of items and return UNKNOWN.
    state           Check a list of state. Return CRITICAL if --critical word found. Else --warning or -- ok. Else unknown will be return.
    alive           Return Warning (-W, --ret_warning) or Critical(-C, --ret_critical) if hypervisor not alive not enabled.
    hosts_connected Return Warning (-W, --ret_warning) or Critical(-C, --ret_critical) if some host can't access to shared storage
    ha_enabled      Return Warning (-W, --ret_warning) or Critical(-C, --ret_critical) if ha not enabled.
    host_alive      Return OK if host number is greather or equal to treshold percent_min. Else return Warning or Critical.
    XXX_usage       Return OK if value lower than treshold percent_max. Else return Warning or Critical.
    snapshots_min   Return OK if (total or lower -p, --period aged) snapshots number is greater than min_snap treshold.
    snapshots_max   Return OK if value is lower than max_snap treshold.
    snapshots_old   Return OK if value is lower than max_days treshold.

<?php
    exit($INT_STATUS["UNKNOWN"]);
}
