<?php
/**
    @file
    @brief IN PROGRESS/FICHIER NON INCLUT : Devrait Permettre de simplifier la récupération de metriques
    @author Julien George
*/
class Monitor{
    private $exec_path = '/usr/local/esia/public_html/plugin/virtcluster/drivers';
    private $authfile = NULL;
    private $driver = NULL;

    function __construct($name,$settings_path){
        if( $name == NULL ){
            if( stripos($settings_path,'xen')){
                $name = 'xenserver';
            }
            else if( stripos($settings_path,'proxmox')){
                $name = 'proxmox';
            }
        }
        $drivers['proxmox']['file'] = 'proxmox/Proxmox.class.php';
        $drivers['proxmox']['class'] = 'ProxmoxCluster';
        $drivers['xenserver']['file'] = 'xenserver/XenServer.class.php';
        $drivers['xenserver']['class'] = 'XenServerCluster';

        if( !isset($drivers[$name])) throw new Exception('Unable to found driver : '.$name);

        $oldpath = get_include_path();
        set_include_path($this->exec_path);
        include_once($drivers[$name]['file']);
        set_include_path($oldpath);

        $settings = file_get_contents($settings_path);
        $settings = json_decode($settings,TRUE);
        $className = $drivers[$name]['class'];
        $this->driver = new $className($settings);
    }

    public static function boolToString($value){
        if( $value === NULL ){
            return 'N/A';
        }
        else if( $value) {
            return 'TRUE';
        }
        else{
            return 'FALSE';
        }
    }

    public static function metricsToArray($usage,$total,$unit){
        $ret['percent'] = $usage/$total*100;
        $ret['total'] = $total;
        $ret['usage'] = $usage;
        $ret['unit'] = $unit;
        return $ret;
    }

    public static function percentToArray($usage,$total,$unit){
        $ret['percent'] = $usage*100;
        $ret['total'] = $total;
        $ret['usage'] = $usage;
        $ret['unit'] = $unit;
        return $ret;
    }

    public static function bdwToArray($in,$out,$unit){
        $retRow['in'] = $in;
        $retRow['out'] = $out;
        $retRow['both'] = $in + $out;
        $retRow['unit'] = $unit;
        return $retRow;
    }

    public function getMetrics($resource=NULL,$id=NULL){
        $ret = array();
        if( strtolower($resource) == 'vm'){
            foreach($this->driver->getVMByName($id) as $vm){
                $retRow = array();
                $retRow['ha'] = self::boolToString($vm->isHA());
                $retRow['ha_settings'] = $vm->getHASettings();
                $retRow['locked'] = self::boolToString($vm->isLocked());
                $retRow['agent'] = self::boolToString($vm->isAgent());
                $retRow['power_state'] = $vm->getState();
                $retRow['cpu_platform'] = $vm->getCpuPlatform();
                $retRow['volumes'] = count($vm->getVolumes());

                $retRow['cpu'] = self::percentToArray($vm->getCpusUsage(),$vm->getCpusTotal(),'nb_vcpu');
                $retRow['ram'] = self::metricsToArray($vm->getMemUsage(),$vm->getMemTotal(),'octets');
                $retRow['net'] = self::bdwToArray( $vm->getNetInUsage(),$vm->getNetOutUsage(),'b/s');

                $retRow['io_disks'] =  self::bdwToArray( $vm->getDiskRead(),$vm->getDiskWrite(),'octets/s');
                $ret[] = $retRow;
            }
        }
        elseif(strtolower($resource) == 'host'){
            $host = $this->driver->getHostByName($id);
            //print_r($host->jsonSerialize());

            $retRow = array();
            $retRow['online'] = self::boolToString($host->isAlive());

            $retRow['cpu_ha'] = self::percentToArray($host->getCpusHA(),$host->getCpusTotal(),'cpu');
            $retRow['cpu'] = self::percentToArray($host->getCpusUsage(),$host->getCpusTotal(),'cpu');
            $retRow['ram_ha'] = self::metricsToArray($host->getMemHA(),$host->getMemTotal(),'octets');
            $retRow['ram'] = self::metricsToArray($host->getMemUsage(),$host->getMemTotal(),'octets');
            $retRow['net'] = self::bdwToArray($host->getNetInUsage(),$host->getNetOutUsage(),'b/s');

            $vm_list = $host->getVMs();
            $retRow['running_vcpu'] = 0;
            $retRow['vm_alive'] = 0;
            $retRow['vm_ha'] = 0;
            foreach($vm_list as $vm){
                if( $vm->isAlive() ) {
                    $retRow['vm_alive']++;
                    $retRow['running_vcpu'] += $vm->getCpusTotal();
                    if( $vm->isHA())     $retRow['vm_ha']++;

                }
            }
            $ret[] = $retRow;
        }
        elseif(strtolower($resource) == 'sr'){
            $storages = $this->driver->getStorageByName($id);

            foreach($storages as $sr){
                $retRow = array();
                print_r($sr->jsonSerialize());

                $host_connected = count($sr->getHosts(TRUE));
                $host_disconnected = count($sr->getHosts(FALSE));

                $retRow['health'] = self::boolToString($host_disconnected==0);
                $retRow['hosts'] = self::metricsToArray($host_connected,$host_connected+$host_disconnected,'nb');

                $size = $sr->getSizeTotal();
                $used = $sr->getSizeUsed();
                $virt = $sr->getSizeVirtual();

                $retRow['used'] = self::metricsToArray($used,$size,'octets');
                $retRow['allowed'] = self::metricsToArray($virt,$size,'octets');

                $retRow['volumes_total'] = 0;
                $retRow['volumes_running'] = 0;

                $vm_alive_list = array();

                $read = 0;
                $write = 0;

                $volumes_list = $sr->getVolumes();
                foreach($volumes_list as $volume){
                    $retRow['volumes_total']++;
                    $vm_alive = FALSE;
                    $read += $volume['io_write'];
                    $write+= $volume['io_read'];

                    foreach($volume['vm'] as $vm){
                        if($vm != NULL && $vm->isAlive()){
                            $vm_alive = TRUE;
                            $vm_alive_list[$vm->getID()]=TRUE;
                        }
                    }
                    if( $vm_alive ) $retRow['volumes_running']++;
                }
                $retRow['vm_running'] = count(array_keys($vm_alive_list));
                //$retRow['io_disk'] = self::bdwToArray($read, $write,'octets/s');
                $retRow['io_disk'] = self::bdwToArray($sr->getDiskRead(),$sr->getDiskWrite(),'octets/s');
            }
            $ret[] = $retRow;
        }
        elseif(strtolower($resource) == 'volumes'){
            foreach($this->driver->getVMByName($id) as $vm){
                $volumes = $vm->getVolumes();
                foreach($volumes as $volume){
                    $used = $volume['size_used'];
                    $total = $volume['size_virt'];

                    $retRow = array();
                    $retRow['parent'] = self::boolToString($volume['parent']!==NULL);
                    $retRow['used'] = self::metricsToArray($used,$total,'octets');
                    $retRow['io_disk'] = self::bdwToArray($volume['io_read'],$volume['io_write'],'octets/s');
                    $ret[] = $retRow;
                }
            }
        }
        else if(strtolower($resource) == 'cluster'){
            $retRow = array();
            $cluster = $this->driver;

            $retRow['HA'] = self::boolToString($cluster->isHA());
            $retRow['clustered'] = self::boolToString($cluster->isCluster());

            $ha = $cluster->getCpusHA();
            $used = $cluster->getCpusUsage();
            $total = $cluster->getCpusTotal();

            $retRow['cpu_ha'] = self::percentToArray($ha,$total,'cpu');
            $retRow['cpu_used'] = self::percentToArray($used,$total,'cpu');

            $ha = $cluster->getMemHA();
            $used = $cluster->getMemUsage();
            $total = $cluster->getMemTotal();

            $retRow['mem_ha'] = self::metricsToArray($ha,$total,'octets');
            $retRow['mem_used'] = self::metricsToArray($used,$total,'octets');

            $hosts  = $cluster->getHosts();
            $hosts_up = 0;
            $hosts_total = 0;
            $vcpus = 0;
            foreach( $hosts as $host){
                if($host->isAlive()){
                    $hosts_up++;
                }
                $hosts_total++;
            }
            $retRow['hosts_up'] = self::metricsToArray($hosts_up,$hosts_total,'hosts');
            $ret[] = $retRow;
        }
        return $ret;
    }
}

$pathfile = $argv[1];
$resource = $argv[2];
$name = NULL;
if( $resource != 'cluster'){
    $name = $argv[3];
}

$monitor = new Monitor(NULL,$pathfile);
print_r($monitor->getMetrics($resource,$name));
