<?php
include_once 'common/HttpClient.php';

/**
    @file
    @brief Permet simplifier l'envoi de requête vers une API Proxmox
    @details
		La classe ProxmoxClient de ce fichier étends la classe HTTPClient en redéfinissant
		certaines méthodes pour simplifier une utilisation avec l'API Proxmox.
    @author Julien George
*/

class ProxmoxClient extends HttpClient
{
	protected $driverSettings;
	protected $request_counter=0;
	protected $connection_recycled = TRUE;
	protected $autoopen;
	protected $ticketfile=NULL;

	public function __construct($settings,$autoopen=FALSE)
	{
		$this->driverSettings = $settings;
		if( isset( $settings['url'] ) ){
			$url = $settings['url'];
			$parse = parse_url($settings['url']);
			$keys = array('scheme','host','port');
			foreach($keys as $key){
				$this->driverSettings[$key] = $parse[$key];
			}
		}
		else
		{
               		$scheme = @$this->driverSettings['scheme'] ? : 'https';
               		$fqdn = @$this->driverSettings['host'];
                	$port = @$this->driverSettings['port'] ? : 8006;
			$url = "$scheme://$fqdn:$port";
		}

		$path_prefix = 'api2/json';
		$url = "$url/$path_prefix";

		parent::__construct($url);
		$this->autoopen=$autoopen;
		if($autoopen){
			$this->open();
		}
	}

	public function open($use_ticket = TRUE)
	{
		$params['realm'] = @$this->driverSettings['proxmox_realm_srv'] ? : 'pam';
		$params['username'] = @$this->driverSettings['login'] ? : 'root';
		$params['password'] = @$this->driverSettings['password'];

		$path_prefix = 'api2/json';

		parent::open();

		$this->ticketfile =  sys_get_temp_dir() .'/pve_'. md5($this->driverSettings['host'].$params['username'].$params['password']);
		if($use_ticket)	$tf_content = @file_get_contents($this->ticketfile);
		else  					$tf_content = FALSE;
		if( $tf_content != FALSE ) $tf_content = json_decode($tf_content,TRUE);
		if( $tf_content == FALSE){
			$this->connection_recycled = FALSE;
			$data = $this->get('access/ticket', $params,FALSE);
			if (empty ($data))
				throw new Exception('Unable to connect : '.$data['ticket']);
			file_put_contents($this->ticketfile,json_encode($data));
		}
		else{
			$data = $tf_content;
		}

		//Le ticket serait valable 2h
		$ticket = $data['ticket'];
		$preventionToken=$data['CSRFPreventionToken'];
		$this->setCookies(TRUE);
		$this->setoptions(CURLOPT_COOKIE,'PVEAuthCookie='.$ticket);

		$header[] =  "CSRFPreventionToken: $preventionToken";
		$this->setoptions(CURLOPT_HTTPHEADER,$header);
	}

	public function execute_request($options = null)
	{
		$data = parent::execute_request($options);

		if ( !in_array($this->getHttpCode(),array(200,201,204)))
		{
			if( $this->getHttpCode() == 401 && $this->ticketfile !=NULL && $this->connection_recycled ){
				unlink($this->ticketfile);
				if( $this->autoopen ){
					$this->close();
					$this->open(FALSE);
					return $this->execute_request($options);
				}
				else{
					throw new Exception('Http error : 401 Usage of expired proxmox ticket');
				}
			}
			throw new Exception('Http error : '.$this->getHttpCode(). " $data");
		}

		$this->request_counter++;

		if (empty ($data))
			return NULL;
		$data = json_decode($data, TRUE);
		if ($data === FALSE)
			throw new Exception('Unable to parse json response');
		return $data['data'];
	}
}
