<?php
class VMwareSoapClient
{
	protected $driverSettings;
	protected $url;
	protected $soapConnection=NULL;
	protected $autoopen;
	protected $serviceContent = NULL;

	public static function resultToOID($data)
	{
	    if( !is_array($data) )
	    {
	        $data = (array) $data;
	    }

	    $ret = array();
	    foreach($data as $key => $row)
	    {
	        if( is_array($row) || is_object($row) )
	        {
	            $childs = self::resultToOID($row);
	            foreach($childs as $kchild => $rchild)
	            {
	                 $ret["$key.$kchild"] = $rchild;
	            }
	        }
	        else
	        {
	            $ret[$key] = $row;
	        }
	    }
	    return $ret;
	}


	public function __construct($settings,$autoopen=FALSE)
	{
		$this->driverSettings = $settings;
		if( isset( $settings['url'] ) ){
			$url = $settings['url'];
			$parse = parse_url($settings['url']);
			$keys = array('host','port');
			foreach($keys as $key){
				$this->driverSettings[$key] = $parse[$key];
			}
		}
		else
		{
			$fqdn = @$this->driverSettings['host'];
			$port = @$this->driverSettings['port'] ? : 443;
			$url = "$scheme://$fqdn:$port";
		}

		$this->url = $url;
		$this->autoopen=$autoopen;
		if($autoopen){
			$this->open();
		}
	}

	public function __destruct()
	{
		if($this->autoopen){
			$this->close();
		}
	}

	public function open()
	{
		if( $this->soapConnection != null ) throw new Exception('Soap resource already opened');

		$stream_context = stream_context_create(array( 'ssl' => array(
		'verify_peer'       => false,
		'verify_peer_name'  => false,
		'allow_self_signed' => true)));

		$this->soapConnection = new SoapClient($this->url.'/sdk/vimService.wsdl',
			array(
				"trace" => 1,
				"location"=>$this->url.'/sdk/',
				"stream_context" => $stream_context
		   )
		);

		$params['username'] = @$this->driverSettings['login'];
		$params['password'] = @$this->driverSettings['password'];

		$serviceContent = $this->getServicecontent();
		$sessionManager = $serviceContent->sessionManager;

		$soapMessage = array(
		"_this"    => new SoapVar($sessionManager->{'_'},  XSD_STRING, $sessionManager->type),
		"userName" => $params['username'], //ou new SoapVar("login", XSD_STRING,'xsd:string');
		"password" => $params['password']  //ou new SoapVar("password", XSD_STRING,'xsd:string');
		);

		$this->soapConnection->Login($soapMessage);
	}

	public function close()
	{
		if( $this->soapConnection == null )
		{
			return;
		}

		$serviceContent = $this->getServicecontent();
		$sessionManager = $serviceContent->sessionManager;

		$soapMessage = array(
			"_this"    => new SoapVar($sessionManager->{'_'},  XSD_STRING, $sessionManager->type)
		);

		$this->soapConnection->Logout($soapMessage);
		$this->soapConnection = NULL;
	}

	public function getSoapConnection()
	{
		if( $this->soapConnection == null )
		{
			throw new Exception('Soap resource must be opened');
		}
		return $this->soapConnection;
	}

	public function getServicecontent()
	{
		$connection = $this->getSoapConnection();

		if( $this->serviceContent == NULL )
		{
			$soapMessage = array(
				"_this" => new SoapVar("ServiceInstance", XSD_STRING, "ServiceInstance")
			);
			$this->serviceContent = $connection->RetrieveServiceContent($soapMessage)->returnval;
		}
		return $this->serviceContent;
	}

	public function getFunctions()
	{
		return $this->getSoapConnection()->__getFunctions();
	}

	public function getTypes()
	{
		return $this->getSoapConnection()->__getTypes();
	}

	public function getLastRequest()
	{
		 return $this->getSoapConnection()->__getLastRequest();
	}

	public function getLastResponse()
	{
		return $this->getSoapConnection()->__getLastResponse();
	}

	private function retrieveProperties($type,$objectSet,$filter=NULL)
	{
		$serviceContent = $this->getServicecontent();
		$connection = $this->getSoapConnection();

		if( empty($filter) )
		{
			$propSet = array('type' => $type,'all'=> 1);
		}
		else
		{
			if( !is_array($filter) )
			{
				$filter = explode(',',$filter);
			}
			$propSet = array('type' => $type,'all'=> 0,'pathSet'=>$filter);
		}

		$params = new stdClass();
		$params->_this = $serviceContent->propertyCollector;
		$params->specSet = array(
			'propSet' => array($propSet),
			'objectSet' => $objectSet,
		);

		$data =  $connection->RetrieveProperties($params);
		return $data;
		$data = $data->returnval;
		if( is_array($data))
		{
			$tmp = array();
			foreach($data as $row)
			{
				$tmp[] = $row->propSet;
			}
			return $tmp;
		}
		else
		{
			return $data->propSet;
		}
	}

	//Retrieves the properties of a set of managed objects identified by id.
	public function getView($type,$listItems,$properties=NULL)
	{
		if( !is_array($listItems) )
		{
			$listItems = explode(',',$listItems);
		}

		$objectsList = array();
		foreach($listItems as $item)
		{
			$objectsList[] = new SoapVar('<objectSet><obj type="'.$type.'">'.$item.'</obj></objectSet>',XSD_ANYXML);
		}
		return $this->retrieveProperties($type,$objectsList,$properties);
	}

	//Retrieves the properties of a set of managed objects.
	public function getViewList($type,$properties=NULL)
	{
		$managedList = self::getManagedEntityTypeList();
		if( !isset($managedList[$type] ) )
		{
			throw new Exception('Unknow managed entity type');
		}
		return $this->getView($type,$properties);
	}

	//Searches the inventory tree for a managed entity that matches the specified entity type. Search begins with root folder unless the begin_entity parameter is specified.
	public function getInventoryTree($type,$properties=NULL)
	{
		$serviceContent = $this->getServicecontent();
		$objectSet['obj'] = $serviceContent->rootFolder;
		$objectSet['skip'] = 0;
		$objectSet['selectSet'] = self::getTraversalList();
		return $this->retrieveProperties($type,$objectSet,$properties);
	}

	public function getPerfsCounter($group=NULL,$name=NULL,$unit=NULL,$rollup = NULL)
	{
		$serviceContent = $this->getServicecontent();
		$objectSet['obj'] = $serviceContent->perfManager;
		$objectSet['skip'] = 0;

		$data = $this->retrieveProperties('PerformanceManager',$objectSet,'perfCounter');
		$data = $data->returnval->propSet;
		$data = $data->val->PerfCounterInfo;

		$ret = array();
		foreach($data as $row)
		{
			if( $name != NULL && $name != $row->nameInfo->key ) continue;
			if( $group != NULL && $group != $row->groupInfo->key ) continue;
			if( $unit != NULL && $unit != $row->unitInfo->key ) continue;
			if( $rollup != NULL && $rollup != $row->rollupType ) continue;

			$retRow['key'] = $row->key;
			$retRow['nameInfo'] = self::parseInfoObject($row->nameInfo);
			$retRow['groupInfo'] = self::parseInfoObject($row->groupInfo);
			$retRow['unitInfo'] = self::parseInfoObject($row->unitInfo);
			$retRow['rollupType'] = $row->rollupType;
			$retRow['statsType'] = $row->statsType;
			$ret[] = $retRow;
		}
		return $ret;
	}

	public static function parseInfoObject($info)
	{
		return array('key'=>$info->key,'label'=>$info->label,'summary'=>$info->summary);
	}

	public function getPerfsData($type,$source,$p_metrics,$samples,$interval=20)
	{
		$serviceContent = $this->getServicecontent();
		$connection = $this->getSoapConnection();

		if( $interval != 20 && $interval != 300 )
		{
			throw new Exception("Incorrect interval type");
		}
/*
		$managedList = self::getManagedEntityTypeList();
		if( !isset($managedList[$type] )
		{
			throw new Exception('Unknow managed entity type');
		}
		$type_value = $managedList[$type];
*/

/*
		if( $type == 'HostSystem' )
		{
			$type_value = 'ha-host';
		}
		else
		{
			$type_value = $type;
			$type = 'VirtualMachine';
		}
*/
		if( !is_array($p_metrics))
		{
			$p_metrics = array(array('counterId'=>$p_metrics,'instance'=>'*'));
		}
		elseif( isset($p_metrics['instance']) )
		{
			$p_metrics = array($p_metrics);
		}
		elseif( !isset($p_metrics[0]['instance'] ) )
		{
			$tmp = array();
			foreach($p_metrics as $row)
			{
				$tmp[] = array('counterId'=>$row,'instance'=>'*');
			}
			$p_metrics = $tmp;
		}

		$metrics = array();
		foreach($p_metrics as $row)
		{
			$metrics[] = new SoapVar($row, SOAP_ENC_OBJECT);
		}


		$perfManager = $serviceContent->perfManager;

		$params = new stdClass();
		$params->_this = new SoapVar($perfManager->{'_'},  XSD_STRING, $perfManager->type);
		$params->querySpec = array (
		    'entity' => new SoapVar('<entity xsi:type="ManagedObjectReference" type="'.$type.'">'.$source.'</entity>',XSD_ANYXML),
		    'maxSample' => $samples,
		    'metricId'=>$metrics,
		    'intervalId' => $interval,
		    'format' => 'csv'
		);
		return $connection->QueryPerf($params);
	}

	private static $traversalList = NULL;
	private static $managedEntityList = NULL;

	public static function getManagedEntityTypeList()
	{
		if( empty(self::$managedEntityList))
		{
			self::$managedEntityList['ClusterComputeResource'] = 'ha-managedCluster';
			self::$managedEntityList['ComputeResource'] = 'ha-computeResource';
			self::$managedEntityList['Datacenter'] = 'ha-datacenter';
			self::$managedEntityList['Datastore'] = 'ha-datastore';
			self::$managedEntityList['DistributedVirtualSwitch'] = 'ha-distributedVirtualSwitch';
			self::$managedEntityList['Folder'] = 'ha-folder';
			self::$managedEntityList['HostSystem'] = 'ha-host';
			self::$managedEntityList['Network'] = 'ha-network';
			self::$managedEntityList['ResourcePool'] = 'ha-resourcePool';
			self::$managedEntityList['VirtualApp'] = 'ha-virtualApp';
			self::$managedEntityList['VirtualMachine'] = '1';
		}
		return self::$managedEntityList;
	}

	public static function getTraversalList()
	{
		if( empty(self::$traversalList) )
		{
			$traversalSpec_list = array();

			//ResourcePool
			$name = 'resourcePoolTraversalSpec';
			$selectSet = array('resourcePoolTraversalSpec','resourcePoolVmTraversalSpec');
			$traversalSpec_list[$name] = new TraversalSpec(
				$name,'ResourcePool','resourcePool',$selectSet
			);

			$name = 'resourcePoolVmTraversalSpec';
			$selectSet = array();
			$traversalSpec_list[$name] = new TraversalSpec(
				$name,'ResourcePool','vm',NULL
			);

			//ComputeResource
			$name = 'computeResourceRpTraversalSpec';
			$selectSet = array('resourcePoolTraversalSpec','resourcePoolVmTraversalSpec');
			$traversalSpec_list[$name] = new TraversalSpec(
				$name,'ComputeResource','resourcePool',$selectSet
			);

			$name = 'computeResourceHostTraversalSpec';
			$selectSet = array();
			$traversalSpec_list[$name] = new TraversalSpec(
				$name,'ComputeResource','host',NULL
			);

			//Datacenter
			$name = 'datacenterHostTraversalSpec';
			$selectSet = array('folderTraversalSpec');
			$traversalSpec_list[$name] = new TraversalSpec(
				$name,'Datacenter','hostFolder',$selectSet
			);

			$name = 'datacenterVmTraversalSpec';
			$selectSet = array('folderTraversalSpec');
			$traversalSpec_list[$name] = new TraversalSpec(
				$name,'Datacenter','vmFolder',$selectSet
			);

			//other
			$name = 'hostVmTraversalSpec';
			$selectSet = array('folderTraversalSpec');
			$traversalSpec_list[$name] = new TraversalSpec(
				$name,'HostSystem','vm',$selectSet
			);

			//Folder
			$name = 'folderTraversalSpec';
			$selectSet = array_keys($traversalSpec_list);
			$traversalSpec_list[$name] = new TraversalSpec(
				$name,'Folder','childEntity', $selectSet
			);


			self::$traversalList = array();
			foreach($traversalSpec_list as $ts)
			{
				self::$traversalList[] = $ts->generateSoapParam();
			}
		}
		return self::$traversalList;
	}
}

class TraversalSpec
{
	protected $properties = array();
	protected $selectSet = array();

	public function __construct($name,$type,$path,$select_set = NULL)
	{
		$this->properties['name'] = $name;
		$this->properties['type'] = $type;
		$this->properties['path'] = $path;
		$this->properties['skip'] = 0;
		if( !empty($select_set ) )
		{
			$this->addSelectSet($select_set);
		}
	}

	public function addSelectSet($name_list)
	{
		if( !is_array($name_list) )
		{
			$name_list = explode(',',$name_list);
		}
		foreach($name_list as $name)
		{
			$this->selectSet[] = new SoapVar(array ('name' => $name), SOAP_ENC_OBJECT, null, null, 'selectSet', null);
		}
	}

	public function generateSoapParam()
	{
		$ret = array_merge($this->properties,$this->selectSet);
    	$ret = new soapvar($ret,SOAP_ENC_OBJECT,'TraversalSpec');
    	return $ret;
	}
}
