<?php
/**
    @file
    @brief Classe héritant de "Model_esia" gérant tous les accès à la DB du webplugin virtualization
    @details
    La classe Model_virtualization de ce fichier contient un ensemble de méthodes statiques
    permettant de configurer un Device de type Virtual cluster ou Hypervisor et/ou
    de récupérer de manière générique une Instance d'un objet Driveur.
*/
$path = $_SERVER['DOCUMENT_ROOT'];
include_once $path.'/Model_adm.php';
include_once 'drivers/manifest.php';

class Model_virtualization extends Model_esia
    {
        //Webplugin
        private static $devices_types = array('cluster'=>'Virtual cluster','hypervisor'=>'Hypervisor');
        private static $webplugin_name = 'WEBP_HYPERVISOR';

        static public function getDevicesType(){
            return array_values(self::$devices_types);
        }

        static public function getClusterDeviceType(){
            return self::$devices_types['cluster'];
        }

        static public function getHypervisorDeviceType(){
            return self::$devices_types['hypervisor'];
        }

        //Driver
        static public function getDriver($dbconnection,$name=null,$type=null){
        $sql_select = 'SELECT * FROM virt_driver';
        $arr = array();
        $where = null;
        if( $name != null )
        {
            $arr[':name'] = $name;
            $where = 'drv_name = :name';
        }
        if( $type != null )
        {
            $arr[':type'] = $type;
            if($where == null)
            {
            	$where = 'drv_type = :type';
            }
            else
            {
            	$where.=' AND drv_type = :type';
            }
        }

        if ( $where != null)
        {
        	$sql_select.= " WHERE $where";
        }

        $query = $dbconnection->prepare($sql_select);
        $query = Model_esia::executeStatement($query,$arr);

        $arr = array();
        while($row=$query->fetch(PDO::FETCH_OBJ))
        {
        	$arr[] = array(
            'id' => $row->drv_id,
            'name' => $row->drv_name,
            'type' => $row->drv_type,
            'path' => $row->drv_path,
            'admin'=> $row->drv_admin_page
            );
        }
        return $arr;
    }

    //Driver Configuration
    static public function getVirtAdvanceConfig($dbconnection,$cluster_select,$type){
        //Search node
        $node = Model_adm::InstanceNode($dbconnection,$cluster_select);

        if( $type != NULL ) {
            $cluster_device_type = self::$devices_types[$type];
            if( $node->getType() != $cluster_device_type ) {
                throw new VirtModelException("Device must be of type '$cluster_device_type'");
            }
        }
        else{
            if( !in_array($node->getType(),self::$devices_types) ) {
                throw new VirtModelException("Device must be of type '".implode(' or ',self::$devices_types)."'");
            }
        }

        //Search $advance_config
        $advance_config_type = Model_adm::getAdvanceNodeConfigType($dbconnection,NULL,self::$webplugin_name);
        $advance_config_type = array_shift($advance_config_type);

        $advance_config = Model_adm::getAdvanceNodeConfig($dbconnection,$node->getID(),NULL,$advance_config_type['id']);

        $ret = array();
        $ret['node'] = $node;
        $ret['adv_type'] = $advance_config_type;
        $ret['adv_config'] = $advance_config;

        if( empty($advance_config)) {
            $ret['adv_config'] = array();
        }
        else{
            $ret['adv_config'] = array_shift($advance_config);
        }
        return $ret;
    }

    static public function setClusterDriver($dbconnection,$cluster_select,$driver_select){
            //Search driver
            $driver_list = self::getDriver($dbconnection);

            $driver_found = array();
            foreach( $driver_list as $driver){
                $driver['name_type'] = $driver['name'] . '-' .$driver['type'];
                if( in_array($driver_select,$driver)){
                    $driver_found[] = $driver;
                }
            }
            if( empty($driver_found)) throw new VirtModelException("Driver $driver_select not found");
            if( count($driver_found) > 1 ) throw new VirtModelException("Multiple $driver_select are possible. You must select one");
            $driver_found = array_shift($driver_found);

            $advance_config = self::getVirtAdvanceConfig($dbconnection,$cluster_select,'cluster');
            $node = $advance_config['node'];
            $advance_config_type =  $advance_config['adv_type'];
            $advance_config = $advance_config['adv_config'];

            //ADD or UPDATE
            if (empty($advance_config)){
                unset($driver_found['name_type']);
                $param['driver']=$driver_found;
                Model_adm::addAdvanceNodeConfig($dbconnection,$node->getID(),$advance_config_type['id'],$param);
            }
            else{
                $param = $advance_config['parameter'];
                $param['driver']=$driver_found;
                Model_adm::updateAdvanceNodeConfig($dbconnection,$advance_config['id'],json_encode($param));
            }
    }
    static public function getClusterDriver($dbconnection,$cluster_select){
        $advance_config = self::getVirtAdvanceConfig($dbconnection,$cluster_select,'cluster');
        $advance_config =  $advance_config['adv_config'];
        return $advance_config['parameter']['driver'];
    }

    //NodeLinkToCluster
    static public function linkNodeToCluster($dbconnection,$node_select,$cluster_select,$force=FALSE){
        $need_add = TRUE;
        if ($force && !empty(Model_virtualization::getClusterNodesLink($dbconnection,$node_id)))
        {
            Model_virtualization::unlinkNode($dbconnection,$node_select,FALSE);
            $need_add = FALSE;
        }

        $node = Model_adm::InstanceNode($dbconnection,$node_select);
        $cluster = Model_adm::InstanceNode($dbconnection,$cluster_select);

        $sql = 'INSERT INTO virt_hypervisor_in(hyperv_device_id,hyperv_cluster_id) VALUES (:hyperv,:cluster)';
        $params[':hyperv'] = $node->getID();
        $params[':cluster'] = $cluster->getID();

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query,$params);

        $advance_config = self::getVirtAdvanceConfig($dbconnection,$cluster_select,'cluster');
        $advance_config_type =  $advance_config['adv_type'];
        $advance_config = $advance_config['adv_config'];

        if( $need_add )
        {
            Model_adm::addAdvanceNodeConfig($dbconnection,$node->getID(),$advance_config_type['id'],array());
        }
    }

    static public function unlinkNode($dbconnection,$node_select,$remove_adv=TRUE){
        $node = Model_adm::InstanceNode($dbconnection,$node_select);

        $sql = 'DELETE FROM virt_hypervisor_in WHERE hyperv_device_id=:hyperv';
        $params[':hyperv'] = $node->getID();

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query,$params);

        $advance_config = self::getVirtAdvanceConfig($dbconnection,$node_select,'hypervisor');
        $advance_config = $advance_config['adv_config'];
        if( $remove_adv )
        {
            Model_adm::deleteAdvanceNodeConfig($dbconnection,$advance_config['id']);
        }
    }

    static public function setHypervisorName($dbconnection,$node_select,$hypervisorName){
        $node = Model_adm::InstanceNode($dbconnection,$node_select);

        $sql = 'UPDATE virt_hypervisor_in SET hyperv_hostname=:hyperv_name WHERE hyperv_device_id=:device_id';
        $params[':device_id'] = $node->getID();
        $params[':hyperv_name'] = $hypervisorName;

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query,$params);
    }

    static public function removeHypervisorName($dbconnection,$node_select){
        self::setHypervisorName($dbconnection,$node_select,NULL);
    }

    static public function getClusterNodesLink($dbconnection,$select=NULL){
        $sql = "SELECT hypervisor.device_id AS device_id,hypervisor.device_name AS device_name,virt_hypervisor_in.hyperv_hostname AS hypervisor_name";
        $sql.= ",cluster.device_id AS cluster_id,cluster.device_name AS cluster_name FROM virt_hypervisor_in";
        $sql.= " INNER JOIN esia_device cluster ON(hyperv_cluster_id = cluster.device_id)";
        $sql.= " INNER JOIN esia_device hypervisor ON(hyperv_device_id = hypervisor.device_id)";

        $params = array();
        if( !empty($select) ){
            if( is_numeric($select) ){
                $sql.= ' WHERE cluster.device_id = :device_id OR hypervisor.device_id = :device_id';
                $params[':device_id'] = $select;
            }
            else{
                $sql.= ' WHERE cluster.device_name = :device_name OR hypervisor.device_name = :device_name';
                $params[':device_name'] = $select;
            }
        }
        $sql.= ' ORDER BY cluster,hypervisor ASC';

        $query = $dbconnection->prepare($sql);
        $query = Model_esia::executeStatement($query,$params);

        $ret = array();
        while($row=$query->fetch(PDO::FETCH_OBJ))
        {
            $ret[] = array(
                'device_id' => $row->device_id,
                'device_name' => $row->device_name,
                'hypervisor_name' => $row->hypervisor_name,
                'cluster_id' => $row->cluster_id,
                'cluster_name' => $row->cluster_name
            );
        }
        return $ret;
    }

    //AuthParam
    static public function getAllClusterNodes($dbconnection,$device_select){
        $list = self::getClusterNodesLink($dbconnection,$device_select);

        $first = reset($list);
        if( $first['device_id'] == $device_select || $first['device_name'] == $device_select )
        {//Si device_select est un hyperviseur, alors liste incomplete. On réinterroge avec l'hyperviseur
            $list = self::getClusterNodesLink($dbconnection,$first['cluster_id']);
        }
        return $list;
    }

    static private function setAuthenticationParams($dbconnection,$action,$device_select,$params){
        $data = self::getVirtAdvanceConfig($dbconnection,$device_select,NULL);
        $adv_id =  $data['adv_config']['id'];
        $node_id = $data['adv_config']['node_id'];

        $keepCurrent = array();
        foreach($params as $key => $value){
            if( $value == NULL ){
                $keepCurrent[] = $key;
            }
        }

        if( !empty($keepCurrent)){
            $current = self::getAuthenticationParams($dbconnection,$device_select);
            $keepCurrent[]='driver';
            foreach($keepCurrent as $key){
                $params[$key] = $current[$key];
            }
        }

        if( !isset($params['driver'])){
            $list = self::getClusterNodesLink($dbconnection,$device_select);
            if( !empty($list) )
            {
                $first = reset($list);
            }
            if( !empty($first['cluster_id']))
            {
                $cluster_config = self::getVirtAdvanceConfig($dbconnection,$first['cluster_id'],'cluster');
                $cluster_config = $cluster_config['adv_config'];

                $params['driver'] = $cluster_config['parameter']['driver']['name'];
            }
            else
            {
                $driver = self::getClusterDriver($dbconnection,$device_select);
                $params['driver'] = $driver['name'];
            }
        }
        Model_adm::updateAuthFile($dbconnection,$adv_id,'virt',json_encode($params),$node_id);
    }

    static public function addAuthenticationParams($dbconnection,$device_select,$url,$login,$password,$driver=NULL){
        $params = array();
        $params['url'] = $url;
        $params['login'] = $login;
        $params['password'] = $password;
        $params['driver'] = $driver;
        self::setAuthenticationParams($dbconnection,'CREATE',$device_select,$params);
    }

    static public function removeAuthenticationParams($dbconnection,$device_select,$hypervisorName=NULL){
        self::setAuthenticationParams($dbconnection,'REMOVE',$device_select,NULL,$hypervisorName);
    }

    static public function updateAuthenticationParams($dbconnection,$device_select,$url,$login=NULL,$password=NULL,$driver=NULL){
        $params = array();
        $params['url'] = $url;
        $params['login'] = $login;
        $params['password'] = $password;
        $params['driver'] = $driver;
        self::setAuthenticationParams($dbconnection,'UPDATE',$device_select,$params);
    }

    static public function getAuthenticationParams($dbconnection,$device_select){
        $data = self::getVirtAdvanceConfig($dbconnection,$device_select,NULL);
        $auth_data = $data['adv_config']['file_content'];
        $auth_data = json_decode($auth_data,TRUE);
        return $auth_data;
    }

    //Todo : Déplacer certaines des méthodes ci-dessous soit dans le driver générique, soit dans le controleur
    private static function getClusterClass($dbconnection,$settings){
        $drivers = getDriversForInstantiate();

        $path = $_SERVER['DOCUMENT_ROOT'];
        $arr = Model_esia::getWebPlugin($dbconnection,true,self::$webplugin_name);

        $drv_name = $settings['driver'];

        if( !isset($drivers[$drv_name]['file']) )
        {
            throw new VirtModelException("Driver $drv_name unknown");
        }

        $oldpath = get_include_path();
        set_include_path($path.$arr[0]['apache_path'].'/drivers/');
        include_once($drivers[$drv_name]['file']);
        set_include_path($oldpath);

        $className = $drivers[$drv_name]['class'];
        $clusterClass = new $className($settings);
        return $clusterClass;
    }

    public static function testDriver($dbconnection,$settings,$driver=NULL){
        if( $driver != NULL ) $settings['driver'] = $driver;
        $cluster = self::getClusterClass($dbconnection,$settings);
        $name = $cluster->getName();
    }

    public static function InstantiateDriver($dbconnection,$node_select){

/*
        //Récupération du driver
        $list = self::getClusterNodesLink($dbconnection,$node_select);
        $first = reset($list);
        $cluster_config = self::getVirtAdvanceConfig($dbconnection,$first['cluster_id'],'cluster');
        $cluster_config = $cluster_config['adv_config'];
        $drv_name = $cluster_config['parameter']['driver']['name'];
*/
        //Récupération des paramètres d'authentication

        $advance_config = self::getVirtAdvanceConfig($dbconnection,$node_select,NULL);
        $advance_config = $advance_config['adv_config'];
        $node_id = $advance_config['node_id'];

        $settings =  $advance_config['file_content'];
        $settings = json_decode($settings, TRUE);
        if(empty($settings))
        {
            throw new VirtModelException("Unable to load authentication settings");
        }

        return self::getClusterClass($dbconnection,$settings);
    }

    public static function objToArray($obj,$fields){
        $ret_array = is_array($obj);
        if( !is_array($obj)) $obj = array($obj);
        if( is_string($fields)) $fields = explode(',',$fields);

        $ret= array();
        foreach($obj as $item){
            $retRow = array();
            foreach($fields as $field){
                $retRow[$field] = $item->getField($field);
            }
            $ret[] = $retRow;
        }
        if( !$ret_array) return array_shift($ret);
        else return $ret;
    }

    public static function getInfraArch($driver){
        $cluster = self::objToArray($driver,'name,master_host,is_cluster');
        $host_list = self::objToArray($driver->getHosts(),'id,name,address,alive');

        $ret['cluster'] = $cluster;
        foreach($host_list as $host){
            $ret['hosts'][$host['id']] = $host;
        };
        ksort($ret['hosts']);
        return $ret;
    }

    public static function getNodeInfraArch($dbconnection,$device_id){
        $driver = self::InstantiateDriver($dbconnection,$device_id);
        return self::getInfraArch($driver);
    }
}

class VirtException extends Exception {};
class VirtModelException extends VirtException {};
?>
